﻿using System;
using System.Threading.Tasks;

using DevExpress.DataAccess;
using DevExpress.DataAccess.Json;
using DevExpress.Utils;
using DevExpress.XtraEditors;
using DevExpress.XtraGrid.Views.Grid;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.gridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            this.gridView.OptionsBehavior.AutoPopulateColumns = true;

            Load += Form_Load;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 폼 로드시 처리하기 - Form_Load(sender, e)

        /// <summary>
        /// 폼 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void Form_Load(object sender, EventArgs e)
        {
            this.gridControl.DataMember = "Customers";

            this.gridView.LoadingPanelVisible = true;

            try
            {
                this.gridControl.DataSource = await GetJSONDataSource();

                this.gridView.BestFitColumns();
            }
            finally
            {
                this.gridView.LoadingPanelVisible = false;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region JSON 데이터 소스 구하기 - GetJSONDataSource()

        /// <summary>
        /// JSON 데이터 소스 구하기
        /// </summary>
        /// <returns>JSON 데이터 소스</returns>
        private async Task<JsonDataSource> GetJSONDataSource()
        {
            JsonDataSource jsonDataSource = new JsonDataSource();

            UriJsonSource uriJsonSource = new UriJsonSource();

            jsonDataSource.JsonSource = new UriJsonSource(new Uri("http://northwind.servicestack.net/customers.json"));

            JsonSchemaNode rootNode         = new JsonSchemaNode("root", null);
            JsonSchemaNode customersNode    = new JsonSchemaNode("Customers"   , true, JsonNodeType.Array);
            JsonSchemaNode companyNameNode  = new JsonSchemaNode("CompanyName" , true, JsonNodeType.Property, typeof(string));
            JsonSchemaNode contactNameNode  = new JsonSchemaNode("ContactName" , true, JsonNodeType.Property, typeof(string));
            JsonSchemaNode contactTitleNode = new JsonSchemaNode("ContactTitle", true, JsonNodeType.Property, typeof(string));
            JsonSchemaNode addressNode      = new JsonSchemaNode("Address"     , true, JsonNodeType.Property, typeof(string));
            JsonSchemaNode cityNode         = new JsonSchemaNode("City"        , true, JsonNodeType.Property, typeof(string));

            customersNode.Nodes.AddRange
            (
                new Node<JsonNode>[]
                {
                    companyNameNode,
                    contactNameNode,
                    contactTitleNode,
                    addressNode,
                    cityNode,
                }
            );

            rootNode.Nodes.Add(customersNode);

            jsonDataSource.Schema = rootNode;

            await jsonDataSource.FillAsync();

            return jsonDataSource;
        }

        #endregion
    }
}